library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity datapath is
    port ( 
		reset: in std_logic; -- resets PC;
		clk: in std_logic; 	 
		-- outputs;
    V: out std_logic; --overflow flag
    C: out std_logic; --carry flag
    N: out std_logic; --negative flag
    Z: out std_logic; --zero flag 
		busA: buffer std_logic_vector(15 downto 0); --bus A	 
		busB: buffer std_logic_vector(15 downto 0); --bus B	 		
		-- inputs;	 
		dataIn: in std_logic_vector(15 downto 0); --bus A	 
		DA: in std_logic_vector(2 downto 0); --address of destination register in register file 
		BA: in std_logic_vector(2 downto 0); --address of source 2 register
		AA: in std_logic_vector(2 downto 0); --address of source 1 register	
		MB: in std_logic; -- mux B	
		FS: in std_logic_vector(3 downto 0); -- FS part of the opcode
		MD: in std_logic; -- mux D	
		RW: in std_logic; -- write of register file 
		MW: in std_logic; -- write of data memory 
		-- "constant in" signal used by datapath;
		zero_fill: in std_logic_vector(15 downto 0)
	);
end datapath;

architecture structural_arch of datapath is	

-- component declarations
component register_file is
	port (
		reset : in std_logic;
		clk : in std_logic;
    write : in std_logic;
    a_address : in std_logic_vector(2 downto 0);
    b_address : in std_logic_vector(2 downto 0);
    d_address : in std_logic_vector(2 downto 0);
    a_data : out std_logic_vector(15 downto 0);
    b_data : out std_logic_vector(15 downto 0);
    d_data : in std_logic_vector(15 downto 0) );
end component;

component function_unit is
    port ( 
		A : in std_logic_vector(15 downto 0);
    B : in std_logic_vector(15 downto 0);
    F : out std_logic_vector(15 downto 0); 
    FS : in std_logic_vector(3 downto 0); 
		V : out STD_LOGIC; 
		C : out STD_LOGIC; 
		N : out STD_LOGIC; 
		Z : out STD_LOGIC );	
end component;

component mux_2to1_16b is
    port (
		mux_select : in std_logic;
    data_a : in std_logic_vector(15 downto 0);
    data_b : in std_logic_vector(15 downto 0);
    data_out : out std_logic_vector(15 downto 0) );
end component;

-- internal signals;
signal busD: std_logic_vector(15 downto 0);	-- out of mux D
signal F: std_logic_vector(15 downto 0);
signal B: std_logic_vector(15 downto 0);

begin

register_file_unit: register_file port map(	  
	reset => reset,
	clk => clk,
  write => RW,
  a_address => AA,
  b_address => BA,
  d_address => DA,
  a_data => busA,
  b_data => B, -- attention; use B not busB; busB is output of mux B instead;
  d_data => busD );

mux_B_unit: mux_2to1_16b port map(	
	mux_select => MB, 
  data_a => B,
  data_b => zero_fill,
  data_out => busB );

ALU_unit: function_unit port map(	
	A => busA,
	B => busB,
  F => F,
  FS => FS,
	V => V,
	C => C,
	N => N,
	Z => Z );
		
mux_D_unit: mux_2to1_16b port map(	
	mux_select => MD, 
  data_a => F,
  data_b => dataIn,
  data_out => busD );
	
end structural_arch;
