--
-- the simplest possible behavioral description of the Function Unit
-- described in Chapter 9 of Mano's book (4th edition); see page 457
-- for the Table describing all supported operations;
--
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
--use IEEE.STD_LOGIC_UNSIGNED.ALL;   
--use IEEE.STD_LOGIC_SIGNED.ALL;
--use IEEE.STD_LOGIC_ARITH.ALL;	-- avoid using this
use IEEE.NUMERIC_STD.ALL; -- use this instead

entity function_unit is
    port ( A : in std_logic_vector(15 downto 0);
      B : in std_logic_vector(15 downto 0);
      F : out std_logic_vector(15 downto 0); 
      FS : in std_logic_vector(3 downto 0); -- function code
      V : out STD_LOGIC; -- overflow (Cn xor Cn-1)
      C : out STD_LOGIC; -- carry out, MSB of result
      N : out STD_LOGIC; -- negative
      Z : out STD_LOGIC );	-- set to 1 if result is zero
end function_unit;

architecture behavioural of function_unit is

signal longA: std_logic_vector(16 downto 0); 
signal longB: std_logic_vector(16 downto 0);
signal result: std_logic_vector(16 downto 0);

begin		

longA <= '0' & A;  
longB <= '0' & B;

process (FS) is
variable var_result: std_logic_vector(16 downto 0);	-- we should avoid using variables;
begin
	V <= '0'; 	 
	
  	case FS is
    when "0000" =>  -- F = A
      	var_result := longA;
    when "0001" =>  -- F = A + 1
      	var_result := std_logic_vector(signed(longA) + 1);
      	if longA(15) = '0' and var_result(15) = '1' then
        	V <= '1';
      	else
        	V <= '0';
      	end if;   
    when "0010" =>  -- F = A + B
      	var_result := std_logic_vector(signed(longA) + signed(longB));
      	if longA(15) = '0' and longB(15) = '0' and var_result(15) = '1' then
        	V <= '1';
      	else
        	V <= '0';
      	end if;     
    when "0011" =>  -- F = A + B + 1
      	var_result := std_logic_vector(signed(longA) + signed(longB) + 1); 
      	if longA(15) = '0' and longB(15) = '0' and var_result(15) = '1' then
        	V <= '1';
      	else
       	 	V <= '0';
      	end if;    
    when "0100" =>  -- F = A + ~B
      	var_result := std_logic_vector(signed(longA) + signed(NOT(longB)));
      	if (longA(15) = '0' and longB(15) = '1' and var_result(15) = '1')
      	or (longA(15) = '1' and longB(15) = '0' and var_result(15) = '0') then
        	V <= '1';
      	else
        	V <= '0';
      	end if;     
    when "0101" =>  -- F = A + ~B + 1
      	var_result := std_logic_vector(signed(longA) + signed(NOT(longB)) + 1);
      	if (longA(15) = '0' and longB(15) = '1' and var_result(15) = '1')
      	or (longA(15) = '1' and longB(15) = '0' and var_result(15) = '0') then
        	V <= '1';
      	else
        	V <= '0';
      	end if; 
    when "0110" =>  -- F = A - 1
      	var_result := std_logic_vector(signed(longA) - 1);
      	if (longA(15) = '1' and var_result(15) = '0') then
        	V <= '1';
      	else
        	V <= '0';
      	end if; 
    when "0111" =>  -- F = A
      	var_result := longA;
    when "1000" =>  -- F = A ^ B
      	var_result := longA AND longB;
    when "1001" =>  -- F = A v B
      	var_result := longA OR longB;  
    when "1010" =>  -- F = A xor B
      	var_result := longA XOR longB;
    when "1011" =>  -- F = ~A
      	var_result := NOT longA;
    when "1100" =>  -- F = B
      	var_result := longB;
    when "1101" =>  -- F = shift right B
      	var_result := std_logic_vector(signed(longB) SRL 1);
    when "1110" =>  -- F = shift left B
      	var_result := std_logic_vector(signed(longB) SLL 1);
    when others =>  
      	var_result := "00000000000000000";
  	end case;
	result <= var_result;
end process;

-- set remaining flags 
F <= result(15 downto 0);
C <= result(16); 
N <= result(15);
Z <= '1' when  result(15 downto 0) = "0000000000000000" else '0'; 
    
end behavioural;
