-- 8 x 16-bit registers
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity register_file is
	port (
		reset : in std_logic;
		clk : in std_logic;
    write : in std_logic;
    a_address : in std_logic_vector(2 downto 0);
    b_address : in std_logic_vector(2 downto 0);
    d_address : in std_logic_vector(2 downto 0);
    a_data : out std_logic_vector(15 downto 0);
    b_data : out std_logic_vector(15 downto 0);
    d_data : in std_logic_vector(15 downto 0) );
end register_file;

architecture behavioral_arch of register_file is

type REGISTER_8x16 is array(0 to 7) of std_logic_vector(15 downto 0);

begin

register_file_process: process (reset, clk,
	a_address, b_address, write, d_address, d_data) is
	variable var_regfile: REGISTER_8x16;
	variable var_read_addr_a: integer;
	variable var_read_addr_b: integer;
	variable var_write_addr_d: integer;
begin
    var_read_addr_a:= conv_integer(a_address);
    var_read_addr_b:= conv_integer(b_address);
    var_write_addr_d:= conv_integer(d_address);    
    if (reset = '1') then
        var_regfile := (others => X"0000");	--reset all registers to 0;
    elsif (rising_edge(clk) and write = '1') then  
		-- synchronous register write should be done on falling edge?
        var_regfile(var_write_addr_d) := d_data;  	
    end if;

    -- asynchronous continuous read of registers;
    -- Note: if component were modelled as a synchronous component
    -- register writes would not be read until the following
    -- rising clock edge (one cycle too late); 
    a_data <= var_regfile(var_read_addr_a); 
    b_data <= var_regfile(var_read_addr_b);	
		
end process; 	 

end behavioral_arch;
